@@ -8,8 +8,9 @@
 # a step change in statistics due to a step stimulus.
 
 
-import numpy
+import numpy,sys
 from NeuroTools import stgen
+from pylab import zeros_like, plot
 
 dt = 10.0
 t = numpy.arange(0,1000.0,dt) 
@@ -33,19 +34,27 @@ tau_r = 1.97
 qrqs = 221.96
 t_stop = 1000.0
 
-trials = 10000
-
+trials = 5000
+print "Running %d trials of %.2f milliseconds" % (trials, t_stop)
 for i in xrange(trials):
+    if i%100==0:
+        print "%d" % i,
+        sys.stdout.flush()
     st = stg.inh_2Dadaptingmarkov_generator(a,bq,tau_s,tau_r,qrqs,t,t_stop,array=True)
-    psth+=numpy.histogram(st,t)[0]
+    psth[1:]+=numpy.histogram(st,t)[0]
+
+print "\n"
 
 # normalize
 
 psth = psth.astype(float)
 psth/= dt*float(trials)/1000.0
 
+# this is for correct 'steps' plotting only
+psth[0] = psth[1]
+
+plot(t,psth,linestyle='steps')
+
 print "Done."
-print "Plotting is left to the user as follows:"
-print "plot(t,psth,linestyle='steps')"
 
 
