@@ -5,6 +5,8 @@
 
 import numpy
 from NeuroTools import stgen
+from pylab import zeros_like, plot
+import sys
 
 dt = 10.0
 t = numpy.arange(0,1000.0,dt) 
@@ -24,14 +26,25 @@ psth = zeros_like(t)
 
 stg = stgen.StGen()
 
-for i in xrange(10000):
+trials = 5000
+tsim = 1000.0
+print "Running %d trials of %.2f milliseconds" % (trials, tsim)
+for i in xrange(trials):
+    if i%100==0:
+        print "%d" % i,
+        sys.stdout.flush()
     st = stg.inh_gamma_generator(a,b,t,1000.0,array=True)
-    psth+=numpy.histogram(st,t)[0]
+    psth[1:]+=numpy.histogram(st,t)[0]
+
+print "\n"
 
 # normalize
 
 psth = psth.astype(float)
 psth/= dt*10000.0/1000.0
 
+# this is for correct 'steps' plotting only
+psth[0] = psth[1]
+
 plot(t,psth,linestyle='steps')
 
