@@ -1,3 +1,4 @@
+# -*- coding: utf-8 -*-
 """
 NeuroTools.io
 ==================
@@ -14,6 +15,8 @@ StandardTextFile   - object used to manipulate text representation of NeuroTools
                      analog signals)
 StandardPickleFile - object used to manipulate pickle representation of NeuroTools objects (spikes or
                      analog signals)
+NestFile           - object used to manipulate raw NEST file that would not have been saved by pyNN
+                     (without headers)
 DataHandler        - object to establish the interface between NeuroTools.signals and NeuroTools.io
 
 All those objects can be used with NeuroTools.signals
@@ -26,7 +29,8 @@ All those objects can be used with NeuroTools.signals
 from NeuroTools import check_dependency
 
 import os, logging, cPickle, numpy
-DEFAULT_BUFFER_SIZE = 10000
+DEFAULT_BUFFER_SIZE = -1
+
 
 
 class FileHandler(object):
@@ -51,6 +55,9 @@ class FileHandler(object):
     def __init__(self, filename):
         self.filename = filename
     
+    def __str__(self):
+        return "%s (%s)" % (self.__class__.__name__, self.filename)
+    
     def write(self, object):
         """
         Write the object to the file. 
@@ -105,26 +112,38 @@ class StandardTextFile(FileHandler):
         the NeuroTools object, if saved in a text file
         """
         cmd = ''
-        f = open(self.filename, 'r')
+        variable = None
+        label    = None
+        f   = open(self.filename, 'r')
         for line in f.readlines():
             if line[0] == '#':
-                cmd += line[1:].strip() + ';'
+                if line[1:].strip().find('variable') != -1:
+                  variable = line[1:].strip().split(" = ")
+                elif line[1:].strip().find('label') != -1:  
+                  label = line[1:].strip().split(" = ")                  
+                else:
+                  cmd += line[1:].strip() + ';'
             else:
                 break
         f.close()
         exec cmd in None, self.metadata
+        if not variable is None:
+          self.metadata[variable[0]] = variable[1]
+        if not variable is None:
+          self.metadata[label[0]] = label[1]
+        
 
     def __fill_metadata(self, object):
         """
         Fill the metadata from those of a NeuroTools object before writing the object
         """
         self.metadata['dimensions'] = str(object.dimensions)
-        if len(object.id_list() > 0):
-            self.metadata['first_id'] = numpy.min(object.id_list())
-            self.metadata['last_id']  = numpy.max(object.id_list())
+        if len(object.id_list) > 0:
+            self.metadata['first_id'] = numpy.min(object.id_list)
+            self.metadata['last_id']  = numpy.max(object.id_list)
         if hasattr(object, "dt"):
             self.metadata['dt']     = object.dt
-
+        
     def __check_params(self, params):
         """
         Establish a control/completion/correction of the params to create an object by 
@@ -136,8 +155,8 @@ class StandardTextFile(FileHandler):
                 params['dt'] = self.metadata['dt']
         if not ('id_list' in params) or (params['id_list'] is None):
             if ('first_id' in self.metadata) and ('last_id' in self.metadata):
-                logging.debug("id_list is infered from the file header")
                 params['id_list'] = range(int(self.metadata['first_id']), int(self.metadata['last_id'])+1)
+                logging.debug("id_list (%d...%d) is infered from the file header" % (int(self.metadata['first_id']), int(self.metadata['last_id'])+1))
             else:
                 raise Exception("id_list can not be infered while reading %s" %self.filename)
         elif isinstance(params['id_list'], int): # allows to just specify the number of neurons
@@ -148,27 +167,33 @@ class StandardTextFile(FileHandler):
             if 'dimensions' in self.metadata:
                 params['dims'] = self.metadata['dimensions']
             else:
-                raise Exception("dims can not be infered while reading %s" %self.filename)
+                params['dims'] = len(params['id_list'])
         return params
-            
+
     def get_data(self, sepchar = "\t", skipchar = "#"):
         """
-        Load data from a text file and returns a list of data
+        Load data from a text file and returns an array of the data
         """
-        myfile = open(self.filename, "r", DEFAULT_BUFFER_SIZE)
+        myfile   = open(self.filename, "r", DEFAULT_BUFFER_SIZE)
         contents = myfile.readlines()
         myfile.close()
-        data = []
-        for line in contents:
-            line = line.strip()
-            if (len(line) != 0):
-                if (line[0] != skipchar):
-                    line = line.split(sepchar)
-                    id    = [int(float(line[-1]))]
-                    id.extend(map(float, line[0:-1]))
-                    data.append(id)
+        data   = []
+        header = True
+        idx    = 0
+        while header and idx < len(contents):
+            if contents[idx][0] != skipchar:
+                header = False
+                break
+            idx += 1
+        for i in xrange(idx, len(contents)):
+            line = contents[i].strip().split(sepchar)
+            id   = [float(line[-1])]
+            id  += map(float, line[0:-1])
+            data.append(id)
+        logging.debug("Loaded %d lines of data from %s" % (len(data), self))
+        data = numpy.array(data, numpy.float32)
         return data
-
+    
     def write(self, object):
         # can we write to the file more than once? In this case, should use seek, tell
         # to always put the header information at the top?
@@ -181,33 +206,34 @@ class StandardTextFile(FileHandler):
         fileobj.close()
 
     def read_spikes(self, params):
-        fileobj = open(self.filename, 'r', DEFAULT_BUFFER_SIZE)
         self.__read_metadata()
-        p = self.__check_params(params)
-        from NeuroTools import signals
-        return signals.SpikeList(self.get_data(), p['id_list'], p['t_start'], p['t_stop'], p['dims'])
+        p    = self.__check_params(params)
+        from NeuroTools.signals import spikes
+        data   = self.get_data()
+        result = spikes.SpikeList(data, p['id_list'], p['t_start'], p['t_stop'], p['dims'])
+        del data
+        return result
 
     def read_analogs(self, type, params):
-        fileobj = open(self.filename, 'r', DEFAULT_BUFFER_SIZE)
         self.__read_metadata()
         p = self.__check_params(params)
-        from NeuroTools import signals
+        from NeuroTools.signals import analogs
         if type == "vm":
-            return signals.VmList(self.get_data(), p['id_list'], p['dt'], p['t_start'], p['t_stop'], p['dims'])
+            return analogs.VmList(self.get_data(), p['id_list'], p['dt'], p['t_start'], p['t_stop'], p['dims'])
         elif type == "current":
-            return signals.CurrentList(self.get_data(), p['id_list'], p['dt'], p['t_start'], p['t_stop'], p['dims'])
+            return analogs.CurrentList(self.get_data(), p['id_list'], p['dt'], p['t_start'], p['t_stop'], p['dims'])
         elif type == "conductance":
             data  = numpy.array(self.get_data())
             if len(data[0,:]) > 2:
-                g_exc = signals.ConductanceList(data[:,[0,1]] , p['id_list'], p['dt'], p['t_start'], p['t_stop'], p['dims'])
-                g_inh = signals.ConductanceList(data[:,[0,2]] , p['id_list'], p['dt'], p['t_start'], p['t_stop'], p['dims'])
+                g_exc = analogs.ConductanceList(data[:,[0,1]] , p['id_list'], p['dt'], p['t_start'], p['t_stop'], p['dims'])
+                g_inh = analogs.ConductanceList(data[:,[0,2]] , p['id_list'], p['dt'], p['t_start'], p['t_stop'], p['dims'])
                 return [g_exc, g_inh]
             else:
-                return signals.ConductanceList(data, p['id_list'], p['dt'], p['t_start'], p['t_stop'], p['dims'])
+                return analogs.ConductanceList(data, p['id_list'], p['dt'], p['t_start'], p['t_stop'], p['dims'])
 
 
 class StandardPickleFile(FileHandler):
-    
+    # There's something kinda wrong with this right now...
     def __init__(self, filename):
         FileHandler.__init__(self, filename) 
         self.metadata = {}
@@ -217,18 +243,19 @@ class StandardPickleFile(FileHandler):
         Fill the metadata from those of a NeuroTools object before writing the object
         """
         self.metadata['dimensions'] = str(object.dimensions)
-        self.metadata['first_id']   = numpy.min(object.id_list())
-        self.metadata['last_id']    = numpy.max(object.id_list())
+        self.metadata['first_id']   = numpy.min(object.id_list)
+        self.metadata['last_id']    = numpy.max(object.id_list)
         if hasattr(object, 'dt'):
             self.metadata['dt']     = object.dt
 
     def __reformat(self, params, object):
         self.__fill_metadata(object)
         if 'id_list' in params and params['id_list'] != None:
-            if isinstance(params['id_list'], int): # allows to just specify the number of neurons
-                params['id_list'] = range(params['id_list'])
-            if params['id_list'] != range(int(self.metadata['first_id']), int(self.metadata['last_id'])+1):
-                object = object.id_slice(params['id_list'])
+            id_list = params['id_list']
+            if isinstance(id_list, int): # allows to just specify the number of neurons
+                params['id_list'] = range(id_list)
+            #if id_list != range(int(self.metadata['first_id']), int(self.metadata['last_id'])+1):
+            #    object = object.id_slice(params['id_list'])
         do_slice = False
         t_start = object.t_start
         t_stop  = object.t_stop
@@ -248,14 +275,150 @@ class StandardPickleFile(FileHandler):
 
     def read_spikes(self, params):
         fileobj = file(self.filename,"r")
-        object = cPickle.load(fileobj)
-        object = self.__reformat(params, object)
+        object  = cPickle.load(fileobj)
+        object  = self.__reformat(params, object)
         return object
         
     def read_analogs(self, type, params):
         return self.read_spikes(params)
 
 
+
+class NestFile(FileHandler):
+    
+    def __init__(self, filename, padding=0, with_time=False, with_gid=True):
+        self.filename  = filename
+        self.metadata  = {}
+        assert (padding >= 0) and (type(padding) == int), "ERROR ! padding should be a positive int"
+        self.padding   = padding
+        self.with_time = with_time
+        self.with_gid  = with_gid
+        self.standardtxtfile = StandardTextFile(filename) 
+    
+    def write(self, object):
+        """
+        Write the object to the file. 
+        
+        Examples:
+            >> handler.write(SpikeListObject)
+            >> handler.write(VmListObject)
+        """
+        return self.standardtxtfile.write(object)
+    
+    def __check_params(self, params):
+        """
+        Establish a control/completion/correction of the params to create an object by 
+        using comparison and data extracted from the metadata.
+        """
+        if 'dt' in params:
+            if params['dt'] is None and 'dt' in self.metadata:
+                logging.debug("dt is infered from the file header")
+                params['dt'] = self.metadata['dt']
+        if params['id_list'] is None:
+            print "WARNING: id_list will be infered based on active cells..."
+        elif isinstance(params['id_list'], int): # allows to just specify the number of neurons
+            params['id_list'] = range(params['id_list'])
+        elif not isinstance(params['id_list'], list):
+            raise Exception("id_list should be an int or a list !")
+        if params['dims'] is None:
+            if 'dimensions' in self.metadata:
+                params['dims'] = self.metadata['dimensions']
+            else:
+                raise Exception("dims can not be infered while reading %s" %self.filename)
+        return params
+    
+    def get_data(self, sepchar = "\t", skipchar = "#"):
+        """
+        Load data from a text file and returns a list of data
+        """
+        if HAVE_TABLEIO:
+            data = TableIO.readTableAsArray(self.filename, skipchar)
+        else:
+            myfile   = open(self.filename, "r", DEFAULT_BUFFER_SIZE)
+            contents = myfile.readlines()
+            myfile.close()
+            data = []
+            header = True
+            idx    = 0
+            while header:
+                if contents[idx][0] != skipchar:
+                    header = False
+                    break
+                idx += 1
+            for i in xrange(idx, len(contents)):
+                line = contents[i].strip().split(sepchar)
+                id   = [float(line[0])]
+                id  += map(float, line[1:])
+                data.append(id)
+        return numpy.array(data)
+
+    def _fix_id_list(self, data, params):
+        print "All gids are shifted by padding", self.padding
+        data[:,0] = numpy.array(data[:,0], int) - self.padding
+        if params['id_list'] is None:
+            params['id_list'] = numpy.unique(data[:,0])
+        return data, params
+
+    def read_spikes(self, params):
+        """
+        Read a SpikeList object from a file and return the SpikeList object, created from the File and
+        from the additional params that may have been provided
+        
+        Examples:
+            >> params = {'id_list' : range(100), 't_stop' : 1000}
+            >> handler.read_spikes(params)
+                SpikeList Object (with params taken into account)
+        """
+        p = self.__check_params(params)
+        from NeuroTools import signals
+        data      = self.get_data()
+        data, p   = self._fix_id_list(data, p)
+        return signals.SpikeList(data, p['id_list'], p['t_start'], p['t_stop'], p['dims'])
+    
+    def read_analogs(self, type, params):
+        p       = self.__check_params(params)
+        data    = self.get_data()
+        data, p = self._fix_id_list(data, p)
+        from NeuroTools.signals import analogs
+        if type == "vm":
+            return analogs.VmList(data, p['id_list'], p['dt'], p['t_start'], p['t_stop'], p['dims'])
+        elif type == "current":
+            return analogs.CurrentList(data, p['id_list'], p['dt'], p['t_start'], p['t_stop'], p['dims'])
+        elif type == "conductance":
+            if len(data[0,:]) > 2:
+                g_exc = analogs.ConductanceList(data[:,[0,1]] , p['id_list'], p['dt'], p['t_start'], p['t_stop'], p['dims'])
+                g_inh = analogs.ConductanceList(data[:,[0,2]] , p['id_list'], p['dt'], p['t_start'], p['t_stop'], p['dims'])
+                return [g_exc, g_inh]
+            else:
+                return analogs.ConductanceList(data, p['id_list'], p['dt'], p['t_start'], p['t_stop'], p['dims'])
+
+
+class PyNNNumpyBinaryFile(FileHandler):
+    
+    def __init__(self, filename):
+        FileHandler.__init__(self, filename)
+        self.fileobj = open(self.filename, 'r')
+        
+    def read_spikes(self, params):
+        from NeuroTools.signals import spikes
+        contents = numpy.load(self.fileobj)
+        spike_data = contents['data'][:, (1,0)]
+        self.metadata = M = {}
+        for k,v in contents['metadata']:
+            M[k] = eval(v)
+        id_list = range(M['first_id'], M['last_id'])
+        t_stop = params['t_stop']
+        # really need to check the agreement between file metadata and
+        # params for all metadata items
+        return spikes.SpikeList(spike_data, id_list, t_start=0.0, t_stop=t_stop,
+                                dims=M['dimensions'])
+        
+    #def read_analogs(self, type, params):
+    #    contents = numpy.load(self.fileobj)
+    #    values, ids = contents['data'].T # need to check the shape first
+    
+
+
 class DataHandler(object):
     """
     Class to establish the interface for loading/saving objects in NeuroTools
@@ -301,6 +464,7 @@ class DataHandler(object):
         ### acccording to the file format.
         ### For the moment, we try the pickle format, and if not working
         ### we assume this is a text file
+        logging.debug("Loading spikes from %s, with parameters %s" % (self.user_file, params))
         return self.user_file.read_spikes(params)
 
 
@@ -325,6 +489,7 @@ class DataHandler(object):
         ### acccording to the file format.
         ### For the moment, we try the pickle format, and if not working
         ### we assume this is a text file
+        logging.debug("Loading analog signal of type '%s' from %s, with parameters %s" % (type, self.user_file, params))
         return self.user_file.read_analogs(type, params)
 
 
