@@ -37,6 +37,9 @@ def gamma_hazard_scipy(x, a, b, dt=1e-4):
         x   - in units of seconds
         a   - dimensionless
         b   - in units of seconds
+
+    See also:
+        inh_gamma_generator
     """
 
     # This algorithm is presently not used by
@@ -70,6 +73,10 @@ def gamma_hazard(x, a, b, dt=1e-4):
         x   - in units of seconds
         a   - dimensionless
         b   - in units of seconds
+
+    See also:
+        inh_gamma_generator
+
     """
 
     # Used by inh_gamma_generator
@@ -78,21 +85,36 @@ def gamma_hazard(x, a, b, dt=1e-4):
     # but the gamma_hazard_scipy above using scipy exhibits numerical problems, as it does not
     # support directly returning the log.
 
-    if not check_dependency('rpy'):
-        raise ImportError("gamma_hazard requires RPy (http://rpy.sourceforge.net/)")
+    if check_dependency('rpy'):
+        from rpy import r
 
-    from rpy import r
+        # scipy.special.gammaincc has numerical problems
+        #Hpre = -log(scipy.special.gammaincc(a,(x-dt)/b))
+        #Hpost = -log(scipy.special.gammaincc(a,(x+dt)/b))
 
-    # scipy.special.gammaincc has numerical problems
-    #Hpre = -log(scipy.special.gammaincc(a,(x-dt)/b))
-    #Hpost = -log(scipy.special.gammaincc(a,(x+dt)/b))
+        # reverting to the good old r.pgamma
+        Hpre = -r.pgamma(x-dt,shape=a,scale=b,lower=False,log=True)
+        Hpost = -r.pgamma(x+dt,shape=a,scale=b,lower=False,log=True)
+        val =  0.5*(Hpost-Hpre)/dt
 
-    # reverting to the good old r.pgamma
-    Hpre = -r.pgamma(x-dt,shape=a,scale=b,lower=False,log=True)
-    Hpost = -r.pgamma(x+dt,shape=a,scale=b,lower=False,log=True)
-    val =  0.5*(Hpost-Hpre)/dt
+        return val
 
-    return val
+    elif check_dependency('rpy2'):
+
+        from rpy2.robjects import r
+
+        # scipy.special.gammaincc has numerical problems
+        #Hpre = -log(scipy.special.gammaincc(a,(x-dt)/b))
+        #Hpost = -log(scipy.special.gammaincc(a,(x+dt)/b))
+
+        # reverting to the good old r.pgamma
+        Hpre = -r.pgamma(x-dt,shape=a,scale=b,lower=False,log=True)[0]
+        Hpost = -r.pgamma(x+dt,shape=a,scale=b,lower=False,log=True)[0]
+        val =  0.5*(Hpost-Hpre)/dt
+
+        return val
+    else:
+        raise ImportError("gamma_hazard requires RPy or RPy2 (http://rpy.sourceforge.net/)")
 
 
     
@@ -132,6 +154,9 @@ class StGen:
         poisson_generator - homogeneous Poisson process
         inh_poisson_generator - inhomogeneous Poisson process (time varying rate)
         inh_gamma_generator - inhomogeneous Gamma process (time varying a,b)
+        inh_adaptingmarkov_generator - inhomogeneous adapting markov process (time varying)
+        inh_2Dadaptingmarkov_generator - inhomogeneous adapting and 
+                                         refractory markov process (time varying)
 
         Continuous time processes:
         --------------------------
@@ -151,6 +176,7 @@ class StGen:
 
         if seed != None:
             self.rng.seed(seed)
+        self.rpy_checked = False
 
     def seed(self,seed):
         """ seed the gsl rng with a given seed """
@@ -177,7 +203,7 @@ class StGen:
             >> gen.poisson_generator(20, 5000, 10000, array=True)
          
         See also:
-            inh_poisson_generator, inh_gamma_generator
+            inh_poisson_generator, inh_gamma_generator, inh_adaptingmarkov_generator
         """
 
         #number = int((t_stop-t_start)/1000.0*2.0*rate)
@@ -228,10 +254,81 @@ class StGen:
         else:
             return spikes
 
+    def gamma_generator(self, a, b, t_start=0.0, t_stop=1000.0, array=False,debug=False):
+        """
+        Returns a SpikeTrain whose spikes are a realization of a gamma process
+        with the given shape a, b and stopping time t_stop (milliseconds). 
+        (average rate will be a*b)
 
+        Note: t_start is always 0.0, thus all realizations are as if 
+        they spiked at t=0.0, though this spike is not included in the SpikeList.
+
+        Inputs:
+            a,b     - the parameters of the gamma process
+            t_start - the beginning of the SpikeTrain (in ms)
+            t_stop  - the end of the SpikeTrain (in ms)
+            array   - if True, a numpy array of sorted spikes is returned,
+                      rather than a SpikeTrain object.
+
+        Examples:
+            >> gen.gamma_generator(10, 1/10., 0, 1000)
+            >> gen.gamma_generator(20, 1/5., 5000, 10000, array=True)
+         
+        See also:
+            inh_poisson_generator, inh_gamma_generator, inh_adaptingmarkov_generator
+        """
+
+        #number = int((t_stop-t_start)/1000.0*2.0*rate)
+        
+        # less wasteful than double length method above
+        n = (t_stop-t_start)/1000.0*(a*b)
+        number = numpy.ceil(n+3*numpy.sqrt(n))
+        if number<100:
+            number = min(5+numpy.ceil(2*n),100)
+        
+        if number > 0:
+            isi = self.rng.gamma(a, b, number)*1000.0
+            if number > 1:
+                spikes = numpy.add.accumulate(isi)
+            else:
+                spikes = isi
+        else:
+            spikes = numpy.array([])
+
+        spikes+=t_start
+        i = numpy.searchsorted(spikes, t_stop)
+
+        extra_spikes = []
+        if i==len(spikes):
+            # ISI buf overrun
+            
+            t_last = spikes[-1] + self.rng.gamma(a, b, 1)[0]*1000.0
+
+            while (t_last<t_stop):
+                extra_spikes.append(t_last)
+                t_last += self.rng.gamma(a, b, 1)[0]*1000.0
+            
+            spikes = numpy.concatenate((spikes,extra_spikes))
+
+            if debug:
+                print "ISI buf overrun handled. len(spikes)=%d, len(extra_spikes)=%d" % (len(spikes),len(extra_spikes))
+
+
+        else:
+            spikes = numpy.resize(spikes,(i,))
+
+        if not array:
+            spikes = SpikeTrain(spikes, t_start=t_start,t_stop=t_stop)
+
+
+        if debug:
+            return spikes, extra_spikes
+        else:
+            return spikes
+            
     def inh_poisson_generator(self, rate, t, t_stop, array=False):
         """
-        Returns a SpikeList whose spikes are a realization of an inhomogeneous 
+        Returns a SpikeTrain whose spikes are a realization of an inhomogeneous 
         poisson process (dynamic rate). The implementation uses the thinning 
         method, as presented in the references.
 
@@ -260,7 +357,7 @@ class StGen:
             >> stgen.inh_poisson_generator(time,sin(time), 1000)
 
         See also:
-            poisson_generator, inh_gamma_generator
+            poisson_generator, inh_gamma_generator, inh_adaptingmarkov_generator
         """
 
         if numpy.shape(t)!=numpy.shape(rate):
@@ -272,7 +369,10 @@ class StGen:
 
         # return empty if no spikes
         if len(ps) == 0:
-            return SpikeTrain(numpy.array([]), t_start=t[0],t_stop=t_stop)
+            if array:
+                return numpy.array([])
+            else:
+                return SpikeTrain(numpy.array([]), t_start=t[0],t_stop=t_stop)
         
         # gen uniform rand on 0,1 for each spike
         rn = numpy.array(self.rng.uniform(0, 1, len(ps)))
@@ -322,7 +422,8 @@ class StGen:
         Devroye, L. (1986). Non-uniform random variate generation. New York: Springer-Verlag.
 
         Examples:
-        
+            See source:trunk/examples/stgen/inh_gamma_psth.py
+
         See also:
             inh_poisson_generator, gamma_hazard
         """
@@ -338,7 +439,10 @@ class StGen:
 
         # return empty if no spikes
         if len(ps) == 0:
-            return SpikeTrain(numpy.array([]), t_start=t[0],t_stop=t_stop)
+            if array:
+                return numpy.array([])
+            else:
+                return SpikeTrain(numpy.array([]), t_start=t[0],t_stop=t_stop)
 
 
         # gen uniform rand on 0,1 for each spike
@@ -378,15 +482,50 @@ class StGen:
 
     # use slow python implementation for the time being
     # TODO: provide optimized C/weave implementation if possible
+      
+    def inh_gamma_generator(self, a, b, t, t_stop, array=False):
+        """
+        Returns a SpikeList whose spikes are a realization of an inhomogeneous gamma process 
+        (dynamic rate). The implementation uses the thinning method, as presented in the 
+        references.
 
+        Inputs:
+            a,b    - arrays of the parameters of the gamma PDF where a[i] and b[i] 
+                     will be active on interval [t[i],t[i+1]]
+            t      - an array specifying the time bins (in milliseconds) at which to 
+                     specify the rate
+            t_stop - length of time to simulate process (in ms)
+            array  - if True, a numpy array of sorted spikes is returned,
+                     rather than a SpikeList object.
 
-    def _inh_gamma_generator_unimplemented(self, a, b, t, t_stop, array=False):
-        raise Exception("inh_gamma_generator is disabled as dependency RPy was not found.")
+        Note: 
+            t_start=t[0]
+            a is a dimensionless quantity > 0, but typically on the order of 2-10. 
+            a = 1 results in a poisson process.
+            b is assumed to be in units of 1/Hz (seconds).
 
-    if check_dependency('rpy'):
-        inh_gamma_generator = _inh_gamma_generator_python
-    else:
-        inh_gamma_generator = _inh_gamma_generator_unimplemented
+        References:
+
+        Eilif Muller, Lars Buesing, Johannes Schemmel, and Karlheinz Meier 
+        Spike-Frequency Adapting Neural Ensembles: Beyond Mean Adaptation and Renewal Theories
+        Neural Comput. 2007 19: 2958-3010.
+        
+        Devroye, L. (1986). Non-uniform random variate generation. New York: Springer-Verlag.
+
+        Examples:
+            See source:trunk/examples/stgen/inh_gamma_psth.py
+
+        See also:
+            inh_poisson_generator, gamma_hazard
+        """
+
+        if not self.rpy_checked:
+            self.have_rpy = check_dependency('rpy') or check_dependency('rpy2')
+            self.rpy_checked = True
+        if self.have_rpy:
+            return self._inh_gamma_generator_python(a, b, t, t_stop, array)
+        else:
+            raise Exception("inh_gamma_generator is disabled as dependency RPy|RPy2 was not found.")
 
 
 
@@ -433,6 +572,8 @@ class StGen:
         Devroye, L. (1986). Non-uniform random variate generation. New York: Springer-Verlag.
 
         Examples:
+            See source:trunk/examples/stgen/inh_2Dmarkov_psth.py
+
         
         See also:
             inh_poisson_generator, inh_gamma_generator, inh_2dadaptingmarkov_generator
@@ -557,6 +698,7 @@ class StGen:
         Devroye, L. (1986). Non-uniform random variate generation. New York: Springer-Verlag.
 
         Examples:
+            See source:trunk/examples/stgen/inh_2Dmarkov_psth.py
         
         See also:
             inh_poisson_generator, inh_gamma_generator, inh_adaptingmarkov_generator
@@ -641,7 +783,7 @@ class StGen:
     def _OU_generator_python(self, dt, tau, sigma, y0, t_start=0.0, t_stop=1000.0, array=False,time_it=False):
         """ 
         Generates an Orstein Ulbeck process using the forward euler method. The function returns
-        an AnalogSignal object
+        an AnalogSignal object.
         
         Inputs:
             dt      - the time resolution in milliseconds of th signal
@@ -650,11 +792,15 @@ class StGen:
             y0      - initial value of the process, at t_start
             t_start - start time in milliseconds
             t_stop  - end time in milliseconds
-            array   - if True, a numpy array of the OU signal is returned,
-                      rather than an AnalogSignal object.
+            array   - if True, the functions returns the tuple (y,t) 
+                      where y and t are the OU signal and the time bins, respectively,
+                      and are both numpy arrays.
         
         Examples:
             >> stgen.OU_generator(0.1, 2, 3, 0, 0, 10000)
+
+        See also:
+            OU_generator_weave1
         """
 
         import time
@@ -681,8 +827,7 @@ class StGen:
         if array:
             return (y,t)
 
-        result = AnalogSignal(y,dt,t_start=0,t_stop=t_stop-t_start)
-        result.time_offset(t_start)
+        result = AnalogSignal(y, dt, t_start, t_stop)
         return result
         
     # use slow python implementation for the time being
@@ -692,7 +837,7 @@ class StGen:
     def _OU_generator_python2(self, dt, tau, sigma, y0, t_start=0.0, t_stop=1000.0, array=False,time_it=False):
         """ 
         Generates an Orstein Ulbeck process using the forward euler method. The function returns
-        an AnalogSignal object
+        an AnalogSignal object.
         
         Inputs:
             dt      - the time resolution in milliseconds of th signal
@@ -701,11 +846,15 @@ class StGen:
             y0      - initial value of the process, at t_start
             t_start - start time in milliseconds
             t_stop  - end time in milliseconds
-            array   - if True, a numpy array of the OU signal is returned,
-                      rather than an AnalogSignal object.
+            array   - if True, the functions returns the tuple (y,t) 
+                      where y and t are the OU signal and the time bins, respectively,
+                      and are both numpy arrays.
         
         Examples:
             >> stgen.OU_generator(0.1, 2, 3, 0, 0, 10000)
+
+        See also:
+            OU_generator_weave1
         """
 
         import time
@@ -732,8 +881,7 @@ class StGen:
         if array:
             return (y,t)
 
-        result = AnalogSignal(y,dt,t_start=0,t_stop=t_stop-t_start)
-        result.time_offset(t_start)
+        result = AnalogSignal(y, dt, t_start, t_stop)
         return result
         
     # use slow python implementation for the time being
@@ -741,12 +889,29 @@ class StGen:
 
 
     def OU_generator_weave1(self, dt,tau,sigma,y0,t_start=0.0,t_stop=1000.0,time_it=False):
-        """ generates an OU process using forward euler
-        dt = resolution
-        tau = correlation time
-        sigma = std dev of process
-        y0 = mean/initial value
-        tsim = how long to simulate
+        """ 
+        Generates an Orstein Ulbeck process using the forward euler method. The function returns
+        an AnalogSignal object.
+
+        OU_generator_weave1, as opposed to OU_generator, uses scipy.weave
+        and is thus much faster.
+        
+        Inputs:
+            dt      - the time resolution in milliseconds of th signal
+            tau     - the correlation time in milliseconds
+            sigma   - std dev of the process
+            y0      - initial value of the process, at t_start
+            t_start - start time in milliseconds
+            t_stop  - end time in milliseconds
+            array   - if True, the functions returns the tuple (y,t) 
+                      where y and t are the OU signal and the time bins, respectively,
+                      and are both numpy arrays.
+        
+        Examples:
+            >> stgen.OU_generator_weave1(0.1, 2, 3, 0, 0, 10000)
+
+        See also:
+            OU_generator
         """
         import scipy.weave
         
@@ -789,8 +954,7 @@ class StGen:
         if array:
             return (y,t)
 
-        result = AnalogSignal(y,dt,t_start=0,t_stop=t_stop-t_start)
-        result.time_offset(t_start)
+        result = AnalogSignal(y,dt,t_start,t_stop)
         return result
 
 
@@ -804,34 +968,10 @@ class StGen:
 
 # TODO fix shotnoise stuff below  ... and write tests
 
-def shotnoise_fromspikes(self,rate,tau,q,num,t):
-    """ 
-    Generate shotnoise
-
-    Inputs:
-        rate - the rate (in Hz) of the shotnoise
-        tau  - the exponential decay of the synapse
-        g    - the quantal increase for a spike
-
-    quantal-increase-"q"-exponential-decay-"tau" synapse
-    and a poisson spike train of "rate" and "num" """
-
-    g = numpy.zeros(numpy.shape(t),float)
-    for i in xrange(num):
-
-        spikes = poisson_generator(rate,t[-1])
-        dg=exp_conv(spikes,t,q,tau)
-        tmp = add(g,dg,g)
-
-    return g
-
-
-
-
 # Operations on spike trains
 
 
-def shotnoise_fromspikes(spike_train,q,tau,dt,array=False, eps = 1.0e-8):
+def shotnoise_fromspikes(spike_train,q,tau,dt=0.1,t_start=None, t_stop=None,array=False, eps = 1.0e-8):
     """ 
     Convolves the provided spike train with shot decaying exponentials
     yielding so called shot noise if the spike train is Poisson-like.  
@@ -842,61 +982,102 @@ def shotnoise_fromspikes(spike_train,q,tau,dt,array=False, eps = 1.0e-8):
       q - the shot jump for each spike
       tau - the shot decay time constant in milliseconds
       dt - the resolution of the resulting shotnoise in milliseconds
+      t_start - start time of the resulting AnalogSignal
+                If unspecified, t_start of spike_train is used
+      t_stop  - stop time of the resulting AnalogSignal
+                If unspecified, t_stop of spike_train is used
       array - if True, returns (shotnoise,t) as numpy arrays, otherwise an AnalogSignal.
       eps - a numerical parameter indicating at what value of 
       the shot kernal the tail is cut.  The default is usually fine.
 
+   Note:
+      Spikes in spike_train before t_start are taken into account in the convolution.
+
    Examples:
       >> stg = stgen.StGen()
       >> st = stg.poisson_generator(10.0,0.0,1000.0)
-      >> g_e = shotnoise_fromspikes(st,2.0,10.0)
+      >> g_e = shotnoise_fromspikes(st,2.0,10.0,dt=0.1)
 
 
    See also:
-      poisson_generator, inh_gamma_generator, OU_generator ...
+      poisson_generator, inh_gamma_generator, inh_adaptingmarkov_generator, OU_generator ...
    """
 
     st = spike_train
 
-    t = numpy.arange(st.t_start,st.t_stop,dt)
+    if t_start is not None and t_stop is not None:
+        assert t_stop>t_start
 
     # time of vanishing significance
     vs_t = -tau*numpy.log(eps/q)
 
+
+    if t_stop == None:
+        t_stop = st.t_stop
+
+    # need to be clever with start time
+    # because we want to take spikes into
+    # account which occured in spikes_times
+    # before t_start
+    if t_start == None:
+        t_start = st.t_start
+        window_start = st.t_start
+    else:
+        window_start = t_start
+        if t_start>st.t_start:
+            t_start = st.t_start
+            
+
+    t = numpy.arange(t_start,t_stop,dt)
+
+
     kern = q*numpy.exp(-numpy.arange(0.0,vs_t,dt)/tau)
 
-    idx = numpy.clip(numpy.searchsorted(t,poisson_train,'right')-1,0,len(t)-1)
+    idx = numpy.clip(numpy.searchsorted(t,st.spike_times,'right')-1,0,len(t)-1)
 
-    a = numpy.zeros(shape(t),float)
+    a = numpy.zeros(numpy.shape(t),float)
 
     a[idx] = 1.0
 
-    y = convolve(a,kern)[0:len(t)]
+    y = numpy.convolve(a,kern)[0:len(t)]
 
-    result = AnalogSignal(y,dt,t_start=0,t_stop=st.t_stop-st.t_start)
-    result.time_offset(st.t_start)
+    if array:
+       signal_t = numpy.arange(window_start,t_stop,dt)
+       signal_y = y[-len(t):]
+       return (signal_y,signal_t)
+
+
+    result = AnalogSignal(y,dt,t_start=0.0,t_stop=t_stop-t_start)
+    result.time_offset(t_start)
+    if window_start>t_start:
+        result = result.time_slice(window_start,t_stop)
     return result
 
 
 
 
 
-def gen_g_add(spikes,tau,q,t,eps = 1.0e-8):
+def _gen_g_add(spikes,q,tau,t,eps = 1.0e-8):
+    """
+
+    spikes is a SpikeTrain object
+
+    """
 
     #spikes = poisson_generator(rate,t[-1])
 
-    gd_s = zeros(shape(t),Float)
+    gd_s = numpy.zeros(t.shape,float)
 
     dt = t[1]-t[0]
 
     # time of vanishing significance
-    vs_t = -tau*log(eps/q)
-    kern = q*exp(-arrayrange(0.0,vs_t,dt)/tau)
+    vs_t = -tau*numpy.log(eps/q)
+    kern = q*numpy.exp(-numpy.arange(0.0,vs_t,dt)/tau)
 
     vs_idx = len(kern)
 
-    idx = clip(searchsorted(t,spikes),0,len(t)-1)
-    idx2 = clip(idx+vs_idx,0,len(gd_s))
+    idx = numpy.clip(numpy.searchsorted(t,spikes.spike_times),0,len(t)-1)
+    idx2 = numpy.clip(idx+vs_idx,0,len(gd_s))
     idx3 = idx2-idx
 
     for i in xrange(len(idx)):
