@@ -3,6 +3,9 @@ Unit tests for the NeuroTools.random module
 
 """
 
+import matplotlib
+matplotlib.use('Agg')
+
 import unittest
 from NeuroTools.random import *
 
@@ -38,4 +41,4 @@ class RandomDistributionTest(unittest.TestCase):
 # ==============================================================================
 if __name__ == '__main__':
     unittest.main()
-    
\ No newline at end of file
+    
